-- COI_AI_Helper
-- Author: HandyVac
-- DateCreated: 7/18/2015 5:49:39 PM
--------------------------------------------------------------

-- The AI is not good at coping with the new glacier and miasmic frost features. Left to itself it will very rarely, if ever remove them.
-- This script will cheat on the AI's behalf to clear glaciers/frost from it's territory at appropriate intervals.

print("*** Loaded COI AI Helper ***");

-------------------------------------------------------------
-- VALUES TO REFERENCE LATER 
-------------------------------------------------------------
local iMinIntervalLength = 15 * Game:GetBuildPercent() / 100; -- These scale with gamespeed, in the same way
local iMaxIntervalLength = 50 * Game:GetBuildPercent() / 100; -- that building/improvement build times do

local iNumPlotsOnMap = Map.GetNumPlots() -1; -- the first plot is plot index 0, not plot index 1
local PlayerGlacierClearInterval = {};

local iThermalMirrorCost = GameInfo.Units["UNIT_THERMAL_MIRROR"].Cost;
local iThawGlacierCost = GameInfo.Builds["BUILD_REMOVE_FOREST"].Cost;
local iThawMiasmicFrostCost = GameInfo.Builds["BUILD_REMOVE_MARSH"].Cost;
local sTechForThermalMirror_Type = GameInfo.Units["UNIT_THERMAL_MIRROR"].PrereqTech;
local iTechForThermalMirror_ID = GameInfo.Technologies[sTechForThermalMirror_Type].ID;
local Unit_ThermalMirror_ID = GameInfo.Units["UNIT_THERMAL_MIRROR"].ID;
local FetGlacier_ID = GameInfo.Features["FEATURE_FOREST"].ID;
local FetMiasmicFrost_ID = GameInfo.Features["FEATURE_MARSH"].ID;
local Res_Titanium_ID = GameInfo.Resources["RESOURCE_TITANIUM"].ID;
-------------------------------------------------------------
-- MAIN FUNCTIONS
-------------------------------------------------------------

function ClearGlaciersForAI (iPlayer)
	local pPlayer = Players[iPlayer];

	-- early out if this is the human player 
	if pPlayer:IsHuman() then
		return false
	end	

	if pPlayer:IsMajorCiv() then
		--print("found an AI major civ");

		-- initialise counter if nil
		if PlayerGlacierClearInterval[iPlayer] == nil then
			PlayerGlacierClearInterval[iPlayer] = (Game.Rand(iMaxIntervalLength - iMinIntervalLength, "rolling for interval length") + iMinIntervalLength);
			print("COI AI Helper: initialising counter for player " .. iPlayer);
		end

		-- is it time to clear a glacier yet?
		if PlayerGlacierClearInterval[iPlayer] <= 0 then
			-- perform a glacier clearance
			print("COI AI Helper: Attempting to perform a feature-thawing action for the AI");
			DoFeatureThawingAction(iPlayer);

			-- reset counter
			PlayerGlacierClearInterval[iPlayer] = (Game.Rand(iMaxIntervalLength - iMinIntervalLength, "rolling for interval length") + iMinIntervalLength);
		else
			-- iterate counter
			print("COI AI Helper: Player " .. iPlayer .. " will do a feature-thawing action in " .. PlayerGlacierClearInterval[iPlayer] .. " turns");
			PlayerGlacierClearInterval[iPlayer] = PlayerGlacierClearInterval[iPlayer] -1;
		end
		
	end
end

-- call this at the start of every players turn
GameEvents.PlayerDoTurn.Add(ClearGlaciersForAI);

---------------------------------------------------------------
-- UTILITY FUNCTIONS
---------------------------------------------------------------

function DoFeatureThawingAction(iPlayer)
	local pPlayer = Players[iPlayer];

	-- first priority, buy a thermal mirror. (If the AI has the energy and resources to buy one)
	if pPlayer:GetEnergy() >= iThermalMirrorCost and pPlayer:HasTech(iTechForThermalMirror_ID) and pPlayer:GetNumResourceTotal(Res_Titanium_ID, true) >= 2 then
		print("  - forcing player " .. iPlayer .. " to buy a thermal mirror");
		BuyThermalMirrorForPlayer(iPlayer);
		return;
	end

	-- second priority, remove a glacier (if the AI has the energy to do the build action)
	if pPlayer:GetEnergy() >= iThawMiasmicFrostCost then
		-- To account for wide/tall empires, do one glacier thaw per city that the AI has
		for i = 1, pPlayer:GetNumCities() do
			if pPlayer:GetEnergy() >= iThawMiasmicFrostCost then
				print("   - forcing player " .. iPlayer .. " to do a Thaw Glacier/Frost action");
				DoWorkerlessThawForPlayer(iPlayer);
			end
		end
		return
	end



	


	-- if none of the above
	print("  - player " .. iPlayer .. " cannot afford to do any thaw action. Doing nothing.");
	return false;
end


function DoWorkerlessThawForPlayer(iPlayer)
	local CandidatePlots = {}
	local pPlayer = Players[iPlayer];
	local iNumValidFeatures = 0;
	
	--print("   - checking map for owned, workable glacier/frost plots");
	-- iterate through all plots on map
	for iCheckedPlot = 0, iNumPlotsOnMap do 
		local pPlot = Map.GetPlotByIndex(iCheckedPlot);
		-- is this plot owned by the player?
		if pPlot:GetOwner() == iPlayer then
			-- is it within workable distance of a city?
			if pPlot:IsPlayerCityRadius(iPlayer) then
				-- does it have a glacier?
				if pPlot:GetFeatureType() == FetGlacier_ID then
					--print("   - found a glacier");
					iNumValidFeatures = iNumValidFeatures +1
					CandidatePlots[iNumValidFeatures] = iCheckedPlot;
				end
				-- does it have miasmic frost?
				if pPlot:GetFeatureType() == FetMiasmicFrost_ID then
					--print("   - found miasmic frost");
					iNumValidFeatures = iNumValidFeatures +1
					CandidatePlots[iNumValidFeatures] = iCheckedPlot;
				end
			end
		end
	end
	
	if iNumValidFeatures == 0 then
		print("   --- no valid glaciers or miasmic frost found. Doing nothing.");
		return;
	else
		-- randomly select one of the valid glaciers
		local roll = ( Game.Rand(iNumValidFeatures, "rolling to pick a glacier to thaw") +1 )
		pPlotToClear = Map.GetPlotByIndex(CandidatePlots[roll]);

		if pPlotToClear:GetFeatureType() == FetGlacier_ID then
			-- remove the feature
			print("   --- removing glacier (one of " .. iNumValidFeatures .. " valid features)");
			pPlotToClear:SetFeatureType(-1);
			-- deduct energy cost from player's treasury
			print("   --- deducting " .. iThawGlacierCost .. " energy");
			pPlayer:ChangeEnergy(0 - iThawGlacierCost);
		else -- it must therefore be frost
			-- remove the feature
			print("   --- removing miasmic frost (one of " .. iNumValidFeatures .. " valid features)");
			pPlotToClear:SetFeatureType(-1);
			-- deduct energy cost from player's treasury
			print("   --- deducting " .. iThawMiasmicFrostCost .. " energy");
			pPlayer:ChangeEnergy(0 - iThawMiasmicFrostCost);
		end
	
	return;
	end
end


function BuyThermalMirrorForPlayer (iPlayer)
	local pPlayer = Players[iPlayer];

	-- iterate through all cities the player owns
	for pCity in pPlayer:Cities() do
		local pCityPlot = pCity:Plot();
		-- unless there's already an unlaunched orbital here, buy a Thermal Mirror.
		if pCityPlot:IsUnlaunchedOrbitalUnit() == false then
			print("   - spawning a thermal mirror");
			pPlayer:InitUnit(Unit_ThermalMirror_ID, pCityPlot:GetX(), pCityPlot:GetY() )
			print("   - deducting " .. iThermalMirrorCost .. " energy");
			pPlayer:ChangeEnergy(0 - iThermalMirrorCost);
			return;
		end
	end

	-- if none of the above
	print("   - could not find a valid city in which to spawn a thermal mirror. Doing nothing.");
	return;
end