-- COI_MapUtilities
-- Author: HandyVac
-- DateCreated: 5/1/2015 4:01:41 PM
--------------------------------------------------------------
print("****** Loaded COI MapUtilities Script *****");
--------------------------------------------------------------
-- VALUES TO REFERENCE LATER
--------------------------------------------------------------
local iNumPlotsOnMap = Map.GetNumPlots() -1; -- the first plot is plot index 0, not plot index 1

local FetIce_ID = GameInfo.Features["FEATURE_ICE"].ID;
local FetFloodPlains_ID = GameInfo.Features["FEATURE_FLOOD_PLAINS"].ID;
local FetGlacier_ID = GameInfo.Features["FEATURE_FOREST"].ID;
local FetFrozenMiasma_ID = GameInfo.Features["FEATURE_MARSH"].ID;

local TerCoastal_ID = GameInfo.Terrains["TERRAIN_COAST"].ID
local TerSnow_ID = GameInfo.Terrains["TERRAIN_SNOW"].ID
local TerTundra_ID = GameInfo.Terrains["TERRAIN_TUNDRA"].ID
local TerPlains_ID = GameInfo.Terrains["TERRAIN_PLAINS"].ID
local TerGrass_ID = GameInfo.Terrains["TERRAIN_GRASS"].ID
local TerDesert_ID = GameInfo.Terrains["TERRAIN_DESERT"].ID

local ImpGenerator_ID = GameInfo.Improvements["IMPROVEMENT_GENERATOR"].ID;
local ImpWorkableMountain_ID = GameInfo.Improvements["IMPROVEMENT_WORKABLE_MOUNTAIN"].ID; 

local Unit_ThermalMirror_ID = GameInfo.Units["UNIT_THERMAL_MIRROR"].ID;

-- this table will record the feature type present on every tile and be updated at end of each round,
-- by comparing it to the currently-present feature we can watch for removed/added features.
local PreviousTurnFeatures = {};

------------------------------------------------------------------
-- MAP SETUP FUNCTIONS
------------------------------------------------------------------
function ConvertMapForCavesOfIce()
	print("-----------------------------------------------");
	print("Starting Caves Of Ice map conversion");
	print("-----------------------------------------------");

	-- early out if this is a savegame being loaded 
	if Game.GetGameTurn() > 0 then
		print("savegame being loaded, no conversion neccesary");
		print("-----------------------------------------------");
		return false;
	end


	-- counters for logging
	local iTotalPackIceAdded = 0;
	local iTotalTundraCooled = 0;
	local iTotalPlainsCooled = 0;
	local iTotalGrassCooled = 0;
	local iTotalDesertCooled = 0;
	local iTotalSnowCooled = 0;
	local iTotalFloodplainsRemoved = 0;
	local iTotalGlacierAdded = 0;
	local iTotalRiverMouthsCleared = 0;
	local iTotalSeaResourceIceCleared = 0;
	local iTotalFeaturesClearedFromExpeditions = 0;
	local iTotalMountainsMadeWorkable = 0;
	local iTotalFeaturesClearedFromResources = 0;
	local iTotalFrostRemovedFromOcean = 0;


	-- iterate through each plot on the map, one at a time	
	print("Checking all plots on map");
	print("-----------------------------------------------");
	-- iterate through every map plot
	for iCounter = 0, iNumPlotsOnMap do 
		local pPlot = Map.GetPlotByIndex(iCounter);
	
			-- ADD COASTAL ICE SHEET

			-- if this is coastal terrain (and not a lake), add ice
			if pPlot:GetTerrainType() == TerCoastal_ID and pPlot:IsLake() == false then
				pPlot:SetFeatureType(FetIce_ID);
				iTotalPackIceAdded = iTotalPackIceAdded +1;
				
				-- if this is a river mouth, remove the ice again
				if IsAdjacentRiver(pPlot) then
					pPlot:SetFeatureType(-1);
					iTotalRiverMouthsCleared = iTotalRiverMouthsCleared +1;
				end

				-- if there is a sea resource here, remove the ice again
				if pPlot:GetResourceType() ~= -1 then
					pPlot:SetFeatureType(-1);
					iTotalSeaResourceIceCleared = iTotalSeaResourceIceCleared +1;
				end

			end

			-- CHANGE TERRAIN TYPES to one tier colder
			-- if snow, add glacier feature 
			if pPlot:GetTerrainType() == TerSnow_ID then
				-- (unless there's a mountain or canyon here)
				if pPlot:CanHaveFeature(FetGlacier_ID) then
					pPlot:SetFeatureType(FetGlacier_ID);
					iTotalGlacierAdded = iTotalGlacierAdded +1;
				end
			end
			
			-- if tundra, make it snow
			if pPlot:GetTerrainType() == TerTundra_ID then
				pPlot:SetTerrainType(TerSnow_ID);
				iTotalTundraCooled = iTotalTundraCooled +1;
			end
			-- if plains, make it tundra
			if pPlot:GetTerrainType() == TerPlains_ID then
				pPlot:SetTerrainType(TerTundra_ID);
				iTotalPlainsCooled = iTotalPlainsCooled +1;
			end
			-- if grassland, make it plains
			if pPlot:GetTerrainType() == TerGrass_ID then
				pPlot:SetTerrainType(TerPlains_ID);
				iTotalGrassCooled = iTotalGrassCooled +1;
			end
			-- if desert, make it... (snow? plains? tundra?)
			if pPlot:GetTerrainType() == TerDesert_ID then
				pPlot:SetTerrainType(TerGrass_ID);
				iTotalDesertCooled = iTotalDesertCooled +1;
				-- if flood plains, remove them
				if pPlot:GetFeatureType() == FetFloodPlains_ID then
					pPlot:SetFeatureType(-1);
					iTotalFloodplainsRemoved = iTotalFloodplainsRemoved +1;
				end
			end
			
			-- If mountain, add a special improvement to make it workable (2 production)
			if pPlot:IsMountain() then
				pPlot:SetImprovementType(ImpWorkableMountain_ID);
				iTotalMountainsMadeWorkable = iTotalMountainsMadeWorkable +1;
			end

			--[[
			-- if there's something an expedition could be built on, remove glaciers and miasmic frost
			if pPlot:GetResourceType() ~= -1 and pPlot:GetFeatureType() ~= -1 then
				-- get the type of resource here
				local StrResourceTypeHere = GameInfo.Resources[pPlot:GetResourceType()].Type;
				-- check whether an expedition could be built on it
				for row in GameInfo.Improvement_ResourceTypes() do
					if row.ResourceType == StrResourceTypeHere and row.ImprovementType == "IMPROVEMENT_EXPEDITION" then
						-- remove any features from this plot
						pPlot:SetFeatureType(-1);
						--print("removed a feature from an expedition site");
						iTotalFeaturesClearedFromExpeditions = iTotalFeaturesClearedFromExpeditions +1;
					end
				end
			end
			--]]
			
			-- if there's any type of resource at all, remove glaciers and miasmic frost
			if pPlot:GetResourceType() ~= -1 and pPlot:GetFeatureType() ~= -1 then
				-- remove the feature
				pPlot:SetFeatureType(-1);
				iTotalFeaturesClearedFromResources = iTotalFeaturesClearedFromResources +1;
			end
			
			
			-- if this is a water tile, remove miasmic frost. (reefs are counted as a type of marsh?)
			if pPlot:IsWater() and pPlot:GetFeatureType() == FetFrozenMiasma_ID then
				pPlot:SetFeatureType(-1);
				print("Found miasmic frost in water, removed it");
			end
			

			-- save data about feature on this plot to the previous turn table
			PreviousTurnFeatures[iCounter] = pPlot:GetFeatureType();
			--print("saved data: plot " .. iCounter .. " has feature " ..  pPlot:GetFeatureType() );

	end


	print("Added Ice to " .. iTotalPackIceAdded .. " coastal plots");
	print("Cleared Ice from " .. iTotalRiverMouthsCleared .. " river mouths");
	print("Cleared Ice from " .. iTotalSeaResourceIceCleared .. " sea resources");
	print("Added " .. iTotalGlacierAdded .. " Glaciers to Snow plots");
	print("Changed " .. iTotalTundraCooled .. " Tundra plots to Snow");
	print("Changed " .. iTotalPlainsCooled .. " Plains plots to Tundra");
	print("Changed " .. iTotalGrassCooled .. " Grassland plots to Plains");
	print("Changed " .. iTotalDesertCooled .. " Desert plots to Grassland");
	print("Removed " .. iTotalFloodplainsRemoved .. " Flood Plains");
	--print("Cleared Glaciers/Miasmic Frost from " .. iTotalFeaturesClearedFromExpeditions .. " Expedition Sites");
	print("Cleared Glaciers/Miasmic Frost from " .. iTotalFeaturesClearedFromResources .. " Land Resources");
	print("Added workable features to " .. iTotalMountainsMadeWorkable .. " Mountian plots");
	print("-----------------------------------------------");

	-- send introduction notification to the player
	local sNotifText =  Locale.ConvertTextKey("TXT_KEY_COI_INTRO_TEXT");
	local sNotifTitle = Locale.ConvertTextKey("TXT_KEY_COI_INTRO_HEADER");
	local pPlayer = Players[Game.GetActivePlayer()];
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sNotifText, sNotifTitle);
end


-- Call this when the spaceflight image with leader quote is shown.
-- i.e. this happens after the map is generated but before it is seen, so terrain graphics are still changeable without needing save & load
Events.SerialEventDawnOfManShow.Add(ConvertMapForCavesOfIce);

-----------------------------------------------------------
-- GAMEPLAY FUNCTIONS
-----------------------------------------------------------


function ProcessThawEffects(iPlayer)
	-- only do this once per round
	if iPlayer > 0 then
		return false;
	end

	print("COI: Processing Thaw Effects");

	-- iterate through all plots on map
	for iCounter = 0, iNumPlotsOnMap do 
		local pPlot = Map.GetPlotByIndex(iCounter);
		local pOrbital =  pPlot:GetOrbitalUnitAbovePlot();

		-- DO THAWS FROM ORBITALS

		-- is there a thermal mirror orbital above this plot?
		if pOrbital ~= nil and pOrbital:GetUnitType() == Unit_ThermalMirror_ID then
			print("   found a thermal mirror in orbit");
			local iTurnsInOrbit = pOrbital:GetTurnsInOrbit();
			local iMaxOrbitTurns = pOrbital:GetTurnsAllowedInOrbit();
			print("   - been here for " .. iTurnsInOrbit .. " turns (max " .. iMaxOrbitTurns .. ")");

			-- determine which tile it should clear
			local iStepSize = iMaxOrbitTurns / 7;
			local pPlotToClear = pPlot;
			local plotX = pPlot:GetX();
			local plotY = pPlot:GetY();

			if iTurnsInOrbit <= iStepSize and iTurnsInOrbit >= (iStepSize/2) then
				-- first do the centre tile
				pPlotToClear = pPlot;
				print("   - clearing centre plot")	
			else
				-- then do the neighbouring tiles, in clockwise order.
				local iDirection = math.floor(iTurnsInOrbit / iStepSize) -1 ;
				pPlotToClear = Map.PlotDirection(plotX, plotY, iDirection)
				print("   - clearing surround plot " .. iDirection);
			end

			-- clear the tile of glaciers
			if pPlotToClear:GetFeatureType() == FetGlacier_ID then
				pPlotToClear:SetFeatureType(-1);
				print("   - orbital removed glacier");
			end
			-- clear the tile of permafrost
			--[[
			if pPlotToClear:GetFeatureType() == FetPermafrost_ID then
				pPlotToClear:SetFeatureType(-1);
				print("   - orbital removed permafrost");
			end
			--]]
			-- clear the tile of miasmic frost
			
			if pPlotToClear:GetFeatureType() == FetFrozenMiasma_ID then
				pPlotToClear:SetFeatureType(-1);
				print("   - orbital removed frozen miasma");
				-- spread miasma in this and surrounding tiles
				--DoMiasmaBomb(pPlotToClear);
			end
		end


		-- SPAWN MIASMA FROM THAWS 

		-- was there frozen miasma here on the previous turn?
		if PreviousTurnFeatures[iCounter] == FetFrozenMiasma_ID then
			-- is that frozen miasma absent this turn?
			if pPlot:GetFeatureType() ~= FetFrozenMiasma_ID then
				print("   frozen miasma was cleared from plot " .. iCounter .. ". Spreading miasma");
				DoMiasmaBomb(pPlot);
			end
		end

		-- was there a glacier here on the previous turn?
		if PreviousTurnFeatures[iCounter] == FetGlacier_ID then
			-- is that glacier absent this turn?
			if pPlot:GetFeatureType() ~= FetGlacier_ID then
				print("   a glacier was removed from plot " .. iCounter );
			end
		end

	

		-- UPDATE SAVED TILE INFO 
		
		-- save data about feature on this plot to the previous turn table
		PreviousTurnFeatures[iCounter] = pPlot:GetFeatureType();
		--print("saved data: plot " .. iCounter .. " has feature " ..  pPlot:GetFeatureType() );
	
	end
end
-- call this at the start of every player's turn
GameEvents.PlayerDoTurn.Add(ProcessThawEffects);

function RemoveFeaturesFromExpeditions()
	--print("checking for newly-created expedition sites");
	-- iterate though all plots on map
	for iCounter = 0, iNumPlotsOnMap do 
		local pPlot = Map.GetPlotByIndex(iCounter);

		
		-- REMOVE FEATURES FROM NEWLY CREATED EXPEDITION SITES

		-- if there's something an expedition could be built on, remove glaciers and miasmic frost
		if pPlot:GetResourceType() ~= -1 and pPlot:GetFeatureType() ~= -1 then
			-- get the type of resource here
			local StrResourceTypeHere = GameInfo.Resources[pPlot:GetResourceType()].Type;
			-- check whether an expedition could be built on it
			for row in GameInfo.Improvement_ResourceTypes() do
				if row.ResourceType == StrResourceTypeHere and row.ImprovementType == "IMPROVEMENT_EXPEDITION" then
					-- remove any features from this plot
					pPlot:SetFeatureType(-1);
					print("removed a feature from an expedition site");
				end
			end
		end
	end
end
-- call this at the end of the human player's turn
Events.ActivePlayerTurnEnd.Add(RemoveFeaturesFromExpeditions)

------------------------------------------------------------
-- UTILITY FUCNCTIONS
------------------------------------------------------------


function IsAdjacentRiver(pPlot)
	-- checks all plots adjacent to input plot: returns true if any of them has a river.
	-- get x and y coords of input plot
	local plotX = pPlot:GetX();
	local plotY = pPlot:GetY();

	-- check adjacent plot in each direction
	for iDirection = 0, 5 do
		local pCheckedPlot = Map.PlotDirection(plotX, plotY, iDirection)
		-- if it's a river, return true
		if pCheckedPlot ~= nil and pCheckedPlot:IsRiver() then
			return true;
		end
	end

	-- if none of the neighbour plots was a river
	return false
end

function DoMiasmaBomb(pPlot)
	-- spread miasma on this plot
	pPlot:SetMiasma(true);
	-- spread miasma on all adjacent plots
	local plotX = pPlot:GetX();
	local plotY = pPlot:GetY();
	for iDirection = 0, 5 do
		local pCheckedPlot = Map.PlotDirection(plotX, plotY, iDirection)
		pCheckedPlot:SetMiasma(true);
	end

	-- Send notification to the player
	local pPlayer = Players[Game.GetActivePlayer()];

	local sNotifText =  Locale.ConvertTextKey("TXT_KEY_THAW_HAS_SPREAD_MIASMA_BODY");
	local sNotifTitle = Locale.ConvertTextKey("TXT_KEY_THAW_HAS_SPREAD_MIASMA_TITLE");
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_INTRUDER_ALERT, sNotifText, sNotifTitle, plotX, plotY);
end