-- GeyserHypocaust_Effect
-- Author: HandyVac
-- DateCreated: 10/30/2015 6:40:14 PM
--------------------------------------------------------------
-- This script will listen for the Geyser Hypocaust wonder being built, then remove all glaciers and miasmic frost features within the surrounding area.

print("****Loaded Geyser Hypocause Effect Script****");

local GeyserHypocaustClass_ID = GameInfo.BuildingClasses["BUILDINGCLASS_GEYSER_HYPOCAUST"].ID;
local GeyserHypocaustType_ID = GameInfo.Buildings["BUILDING_GEYSER_HYPOCAUST"].ID;
local iEffectRange = 3;
local FetGlacier_ID = GameInfo.Features["FEATURE_FOREST"].ID;
local FetFrozenMiasma_ID = GameInfo.Features["FEATURE_MARSH"].ID;

local EffectDone = 0;

function GeyserHypocaustEffect (CurrentPlayerID)
	local pPlayer = Players[CurrentPlayerID];

	-- Early out if the effect has already happened
	if EffectDone == 1 then
		return false;
	end
	

	print("Checking for Geyser Hypocaust wonder");
	-- does the wonder exist anywhere?
	if Game.GetBuildingClassCreatedCount( GeyserHypocaustClass_ID ) > 0 then
		print("Somebody has built the Geyser Hypocaust. Checking who and where.")

		-- cycle through this player's cities to find which one it's in
		for pCity in pPlayer:Cities() do
			if pCity:IsHasBuilding(GeyserHypocaustType_ID) then
				print("Player " .. CurrentPlayerID ..  "owns the Geyser Hypocaust in " .. pCity:GetName() );
				-- Do Wonder Effect

				-- loop through all hexes within square grid of size range

				for gridX = -iEffectRange, iEffectRange do
					for gridY = -iEffectRange, iEffectRange do
						-- check whether this plot is within range (i.e. a circle of radius iEffectRange)
						local pPlot = Map.PlotXYWithRangeCheck(pCity:GetX(), pCity:GetY(), gridX, gridY, iEffectRange);
						if pPlot then
							-- is there a glacier or frost here?
							if pPlot:GetFeatureType() == FetGlacier_ID or pPlot:GetFeatureType() == FetFrozenMiasma_ID then
								-- remove it
								pPlot:SetFeatureType(-1);

								-- flag this as done, so the effect only happens once.
								EffectDone = 1;
							end
						end
					end
				end
				print("...removed all glaciers and miasmic frost within " .. iEffectRange .. " tiles of " .. pCity:GetName() );

			end
		end

	end
end

-- call this at start of every player's turn
GameEvents.PlayerDoTurn.Add(GeyserHypocaustEffect);